@@ -18,6 +18,8 @@
     Boston, MA 02110-1301, USA.
 */
 
+#include "kio_nfs.h"
+
 #include <config-runtime.h>
 
 #include <sys/types.h>
@@ -55,10 +57,8 @@
 #include <kio/global.h>
 #include <iostream>
 
-#include "nfs_prot.h"
 #define fhandle _fhandle
 #include "mount.h"
-#include "kio_nfs.h"
 
 #define MAXHOSTLEN 256
 
@@ -154,8 +154,10 @@ static void createVirtualDirEntry(UDSEntry & entry)
 static void stripTrailingSlash(QString& path)
 {
    //if (path=="/") return;
-   if (path=="/") path="";
-   else if (path[path.length()-1]=='/') path.truncate(path.length()-1);
+   if (path == QLatin1String("/"))
+       path = "";
+   else if (path.endsWith(QLatin1Char('/')))
+       path.truncate(path.length()-1);
 }
 
 static void getLastPart(const QString& path, QString& lastPart, QString& rest)
