@@ -56,11 +56,16 @@ private:
     bool createSubThumbnail(QImage& thumbnail, const QString& filePath,
                             int segmentWidth, int segmentHeight);
 
+    /**
+     * Scales down the image \p img in a way that it fits into the
+     * given maximum width and height.
+     */
+    void scaleDownImage(QImage& img, int maxWidth, int maxHeight);
+
 private:
     QString m_mimeType;
     int m_width;
     int m_height;
-    bool m_keepAspectRatio;
     int m_iconSize;
     int m_iconAlpha;
     // Thumbnail creators
@@ -70,6 +75,7 @@ private:
     QStringList m_enabledPlugins;
     QSet<QString> m_propagationDirectories;
     QString m_thumbBasePath;
+    qint64 m_maxFileSize;
 };
 
 #endif
